@@ -1,6 +1,7 @@
 import inspect
 import json
 import logging
+import time
 from concurrent.futures.thread import ThreadPoolExecutor
 from logging import Logger
 from queue import Queue, Empty
@@ -9,6 +10,7 @@ from threading import Lock, Event
 from typing import Optional, Callable, List, Union
 
 from slack_sdk.errors import SlackApiError, SlackClientError
+from slack_sdk.proxy_env_variable_loader import load_http_proxy_from_env
 from slack_sdk.socket_mode.builtin.connection import Connection, ConnectionState
 from slack_sdk.socket_mode.interval_runner import IntervalRunner
 from slack_sdk.web import WebClient
@@ -29,6 +31,7 @@ class RTMClient:
 
     current_session: Optional[Connection]
     current_session_state: Optional[ConnectionState]
+    wss_uri: Optional[str]
 
     message_queue: Queue
     message_listeners: List[Callable[["RTMClient", dict], None]]
@@ -53,7 +56,7 @@ class RTMClient:
         timeout: int = 30,
         base_url: str = WebClient.BASE_URL,
         headers: Optional[dict] = None,
-        ping_interval: int = 10,
+        ping_interval: int = 5,
         concurrency: int = 10,
         logger: Optional[logging.Logger] = None,
         on_message_listeners: Optional[List[Callable[[str], None]]] = None,
@@ -75,6 +78,10 @@ class RTMClient:
         self.headers = headers
         self.ping_interval = ping_interval
         self.logger = logger or logging.getLogger(__name__)
+        if self.proxy is None or len(self.proxy.strip()) == 0:
+            env_variable = load_http_proxy_from_env(self.logger)
+            if env_variable is not None:
+                self.proxy = env_variable
 
         self.web_client = web_client or WebClient(
             token=self.token,
@@ -87,6 +94,7 @@ class RTMClient:
         )
 
         self.on_message_listeners = on_message_listeners or []
+
         self.on_error_listeners = on_error_listeners or []
         self.on_close_listeners = on_close_listeners or []
 
@@ -95,14 +103,22 @@ class RTMClient:
         self.ping_pong_trace_enabled = ping_pong_trace_enabled
 
         self.message_queue = Queue()
-        self.message_listeners = []
+
+        def goodbye_listener(_self, event: dict):
+            if event.get("type") == "goodbye":
+                message = "Got a goodbye message. Reconnecting to the server ..."
+                self.logger.info(message)
+                self.connect_to_new_endpoint(force=True)
+
+        self.message_listeners = [goodbye_listener]
         self.socket_mode_request_listeners = []
 
         self.current_session = None
         self.current_session_state = ConnectionState()
         self.current_session_runner = IntervalRunner(
-            self._run_current_session, 0.5
+            self._run_current_session, 0.1
         ).start()
+        self.wss_uri = None
 
         self.current_app_monitor_started = False
         self.current_app_monitor = IntervalRunner(
@@ -176,20 +192,25 @@ class RTMClient:
             api_response = self.web_client.rtm_connect()
             return api_response["url"]
         except SlackApiError as e:
-            self.logger.error(f"Failed to retrieve WSS URL: {e}")
-            raise e
+            if e.response["error"] == "ratelimited":
+                delay = int(e.response.headers.get("Retry-After", "30"))  # Tier1
+                self.logger.info(f"Rate limited. Retrying in {delay} seconds...")
+                time.sleep(delay)
+                # Retry to issue a new WSS URL
+                return self.issue_new_wss_url()
+            else:
+                # other errors
+                self.logger.error(f"Failed to retrieve WSS URL: {e}")
+                raise e
 
     def connect_to_new_endpoint(self, force: bool = False):
         """Acquires a new WSS URL and tries to connect to the endpoint."""
-        try:
-            self.connect_operation_lock.acquire(blocking=True, timeout=5)
+        with self.connect_operation_lock:
             if force or not self.is_connected():
                 self.logger.info("Connecting to a new endpoint...")
                 self.wss_uri = self.issue_new_wss_url()
                 self.connect()
                 self.logger.info("Connected to a new endpoint...")
-        finally:
-            self.connect_operation_lock.release()
 
     def connect(self):
         """Starts talking to the RTM server through a WebSocket connection"""
@@ -197,9 +218,13 @@ class RTMClient:
             self.bot_id = self.web_client.auth_test()["bot_id"]
 
         old_session: Optional[Connection] = self.current_session
+        old_current_session_state: ConnectionState = self.current_session_state
 
-        self.current_session = Connection(
-            url=self.issue_new_wss_url(),
+        if self.wss_uri is None:
+            self.wss_uri = self.issue_new_wss_url()
+
+        current_session = Connection(
+            url=self.wss_uri,
             logger=self.logger,
             ping_interval=self.ping_interval,
             trace_enabled=self.trace_enabled,
@@ -212,12 +237,16 @@ class RTMClient:
             on_close_listener=self.run_all_close_listeners,
             connection_type_name="RTM",
         )
-        self.current_session.connect()
-        self.auto_reconnect_enabled = self.default_auto_reconnect_enabled
+        current_session.connect()
 
+        if old_current_session_state is not None:
+            old_current_session_state.terminated = True
         if old_session is not None:
             old_session.close()
-            self.current_session_state.terminated = True
+
+        self.current_session = current_session
+        self.current_session_state = ConnectionState()
+        self.auto_reconnect_enabled = self.default_auto_reconnect_enabled
 
         if not self.current_app_monitor_started:
             self.current_app_monitor_started = True
@@ -348,23 +377,24 @@ class RTMClient:
             listener(code, reason)
 
     def _run_current_session(self):
-        try:
-            if self.current_session is not None and self.current_session.is_active():
+        if self.current_session is not None and self.current_session.is_active():
+            session_id = self.session_id()
+            try:
                 self.logger.info(
                     "Starting to receive messages from a new connection"
-                    f" (session id: {self.session_id()})"
+                    f" (session id: {session_id})"
                 )
                 self.current_session_state.terminated = False
                 self.current_session.run_until_completion(self.current_session_state)
                 self.logger.info(
                     "Stopped receiving messages from a connection"
-                    f" (session id: {self.session_id()})"
+                    f" (session id: {session_id})"
+                )
+            except Exception as e:
+                self.logger.exception(
+                    "Failed to start or stop the current session"
+                    f" (session id: {session_id}, error: {e})"
                 )
-        except Exception as e:
-            self.logger.exception(
-                "Failed to start or stop the current session"
-                f" (session id: {self.session_id()}, error: {e})"
-            )
 
     def _monitor_current_session(self):
         if self.current_app_monitor_started:
